#include "stdafx.h"
#include "degrees.h"
#include "cylinder.h"
#include "RegionData.h"


/****************************************************************************
*                             Cylinder::Cylinder
* Result: Constructor
* 
* Effect: 
*       Initializes values
****************************************************************************/

Cylinder::Cylinder()
    {
     lightAngle = 0;
     darkColor = RGB(0,0,0);
     lightColor = RGB(255,255,255);
     height = 0;
     width = 0;
     eyeAngle = 0;
     UseMatrix = FALSE;
     Centroid = CPoint(0,0);
    } // Cylinder::Cylinder

/****************************************************************************
*                             Cylinder::SetHeight
* Inputs:
*       int height: Height
* Result: void
* 
* Effect: 
*       Sets the height of the cylinder
****************************************************************************/

void Cylinder::SetHeight(int H)
    {
     height = H;
    } // Cylinder::SetHeight

/****************************************************************************
*                             Cylinder::SetWidth
* Inputs:
*       int w: Width
* Result: void
* 
* Effect: 
*       Sets the cylinder width
****************************************************************************/

void Cylinder::SetWidth(int w)
    {
     width = w;
    } // Cylinder::SetWidth

/****************************************************************************
*                             Cylinder::GetHeight
* Result: int
*       Current height in logical units
****************************************************************************/

int Cylinder::GetHeight()
    {
     return height;
    } // Cylinder::GetHeight

/****************************************************************************
*                             Cylinder::GetWidth
* Result: int
*       Width in logical units
****************************************************************************/

int Cylinder::GetWidth()
    {
     return width;
    } // Cylinder::GetWidth

/****************************************************************************
*                              Cylinder::SetSize
* Inputs:
*       CSize sz;
* Result: void
* 
* Effect: 
*       Sets the size
****************************************************************************/

void Cylinder::SetSize(CSize sz)
    {
     width = sz.cx;
     height = sz.cy;
    } // Cylinder::SetSize

/****************************************************************************
*                              Cylinder::GetSize
* Result: CSize
*       Width,height
****************************************************************************/

CSize Cylinder::GetSize()
    {
     return CSize(width, height);
    } // Cylinder::GetSize

/****************************************************************************
*                           Cylinder::SetLightingAngle
* Inputs:
*       int degrees: Light angle (-90..90)
* Result: void
* 
* Effect: 
*       Sets the light angle
* Notes:
*       Out of range values will be ignored
****************************************************************************/

void Cylinder::SetLightingAngle(int degrees)
    {
     if(degrees < -90)
        degrees = -90;
     else
     if(degrees > 90)
        degrees = 90;

     lightAngle = degrees;
    } // Cylinder::SetLightingAngle

/****************************************************************************
*                           Cylinder::GetLightingAngle
* Result: int
*       Lighting angle
****************************************************************************/

int Cylinder::GetLightingAngle()
    {
     return lightAngle;
    } // Cylinder::GetLightingAngle

/****************************************************************************
*                            Cylinder::SetEyeAngle
* Inputs:
*       int degrees: Angle of eye
* Result: void
* 
* Effect: 
*       Sets the eye angle
****************************************************************************/

void Cylinder::SetEyeAngle(int degrees)
    {
     if(degrees < -45)
        degrees = -45;
     else
     if(degrees > 45)
        degrees = 45;
 
     eyeAngle = degrees;
    } // Cylinder::SetEyeAngle

/****************************************************************************
*                            Cylinder::GetEyeAngle
* Result: int
*       Eye angle in degrees
****************************************************************************/

int Cylinder::GetEyeAngle()
    {
     return eyeAngle;
    } // Cylinder::GetEyeAngle

/****************************************************************************
*                             Cylinder::SetColor
* Inputs:
*       COLORREF dk: Dark color
*       COLORREF lt: Light color
* Result: void
* 
* Effect: 
*       Sets the colors
****************************************************************************/

void Cylinder::SetColor(COLORREF dk, COLORREF lt)
    {
     darkColor = dk;
     lightColor = lt;
    } // Cylinder::SetColor

/****************************************************************************
*                             Cylinder::GetColor
* Inputs:
*       COLORREF & dk:
*       COLORREF & lt:
* Result: void
* 
* Effect: 
*       Returns the colors
****************************************************************************/

void Cylinder::GetColor(COLORREF & dk, COLORREF & lt)
    {
     dk = darkColor;
     lt = lightColor;
    } // Cylinder::GetColor

/****************************************************************************
*                           Cylinder::GetLightColor
* Result: COLORREF
* 
****************************************************************************/

COLORREF Cylinder::GetLightColor()
    {
     return lightColor;
    } // Cylinder::GetLightColor

/****************************************************************************
*                           Cylinder::GetDarkColor
* Result: COLORREF
* 
****************************************************************************/

COLORREF Cylinder::GetDarkColor()
    {
     return darkColor;
    } // Cylinder::GetDarkColor

/****************************************************************************
*                           Cylinder::SetLightColor
* Inputs:
*       COLORREF c:
* Result: void
* 
* Effect: 
*       Sets the light color
****************************************************************************/

void Cylinder::SetLightColor(COLORREF c)
    {
     lightColor = c;
    } // Cylinder::SetLightColor

/****************************************************************************
*                           Cylinder::SetDarkColor
* Inputs:
*       COLORREF c:
* Result: void
* 
* Effect: 
*       Sets the dark color
****************************************************************************/

void Cylinder::SetDarkColor(COLORREF c)
    {
     darkColor = c;
    } // Cylinder::SetDarkColor

/****************************************************************************
*                             Cylinder::SetMatrix
* Inputs:
*       XFORM * M: matrix to store, or NULL
* Result: void
* 
* Effect: 
*       Sets the mode.  If the matrix is NULL, the use of the matrix will
*       be ignored
****************************************************************************/

void Cylinder::SetMatrix(XFORM * M)
    {
     if(M == NULL)
        { /* clear it */
         UseMatrix = FALSE;
         return;
        } /* clear it */
     else
        { /* use it */
         UseMatrix = TRUE;
         Matrix = *M;
        } /* use it */
    } // Cylinder::SetMatrix

/****************************************************************************
*                             Cylinder::GetMatrix
* Inputs:
*       XFORM & M: Matrix to set
* Result: BOOL
*       TRUE if the matrix is set, FALSE if the matrix is not set
* Effect: 
*       Sets the matrix to the selected transformation matrix
****************************************************************************/

BOOL Cylinder::GetMatrix(XFORM & M)
    {
     if(!UseMatrix)
        return FALSE;
     M = Matrix;
     return TRUE;
    } // Cylinder::GetMatrix

/****************************************************************************
*                            Cylinder::SetPos
* Inputs:
*       CPoint pt: A point which is the centroid of the cylinder
* Result: void
* 
* Effect: 
*       Sets the desired initial point
****************************************************************************/

void Cylinder::SetPos(CPoint pt)
    {
     Centroid = pt;
    } // Cylinder::SetPos

/****************************************************************************
*                              Cylinder::SetPos
* Inputs:
*       int x: x-position
*       int y: y-position
* Result: void
* 
* Effect: 
*       Sets the position of the centroid
****************************************************************************/

void Cylinder::SetPos(int x, int y)
    {
     SetPos(CPoint(x, y));
    } // Cylinder::SetPos

/****************************************************************************
*                              Cylinder::GetPos
* Result: CPoint
*       Point of center of cylinder
****************************************************************************/

CPoint Cylinder::GetPos()
    {
     return Centroid;
    } // Cylinder::GetPos

/****************************************************************************
*                               Cylinder::Draw
* Inputs:
*       CDC & dc: DC to draw to
* Result: void
* 
* Effect: 
*       Draws the cylinder.  The center of the cylinder is the (0,0) coordinate
*       point in the coordinate system.
* Notes:
*       The DC must be modified to have the window origin set at the point
*       where the cylinder should be drawn
****************************************************************************/

void Cylinder::Draw(CDC & dc)
    {
     int save = dc.SaveDC();
     if(UseMatrix)
        { /* use transformation matrix */
         ::SetGraphicsMode(dc, GM_ADVANCED);
         VERIFY(::SetWorldTransform(dc, &Matrix));
        } /* use transformation matrix */

     dc.SetViewportOrg(Centroid);

     CPoint pt(-(width / 2), -(height / 2) );

     /*================================================================
       Compute the gradient fill based on the light angle
       We have two segments: dark-to-light and light-to-dark

       What we want to do is compute the x-axis value at which the
       light part maximizes

                                   d + (r * sin(theta))
                                   |
                               d   v
       ----- -90 --------------+---+------------ +90 ------------------x
                   --          |\  |      --
                      --       | \ |   --
                         --    |  \|--
                            --.|.--\
                               0    \
                                     theta=30
       ================================================================*/

     double theta = DEGREES_TO_RADIANS((double)lightAngle);
     double xoffset = ((double)width / 2.0) * sin(theta);

     // The distance dx is the distance from the center line to
     // the point of peak brightness.  Note that dx could be negative
     int dx = (int)xoffset;

     // Compute the x-coordinate of the peak of the brightness
     int peak = pt.x + (width / 2) + dx;

     //================
     //      |---#-------|
     //  :-------#-------|
     //================

     // Assume for the moment that the width we are going to draw
     // is the actual width of the image
     int left = pt.x;
     int right = pt.x + width;

     // Now adjust either the right or the left so that the new
     // point places the peak in the center of the area computed,
     // that is, peak == (right - left) / 2
     // after we adjust left and right

     if(dx < 0)
        left = peak - (right - peak);
     else
        right = peak + (peak - left);
 
     // Compute the size of the endcap. Start by assuming
     // both endcaps are "flat", that is, of 0 height
     // These two elements will be used to draw the ellipse
     // for each end

     CRect top   (pt.x, pt.y,          pt.x + width, pt.y);
     CRect bottom(pt.x, pt.y + height, pt.x + width, pt.y + height);

     // Now "rotate" the endcap by the amount based on the eye angle
     double rotate = DEGREES_TO_RADIANS(eyeAngle);

     // Compute the size of the upper curve and the size of the
     // lower curve of the endcaps.  Remember that in MM_TEXT
     // mode (the coordinates we are using) the y-axis
     // increases downward
     int dy = (int) (fabs((double)(width / 2) * sin(rotate)));

     // Compute the dimensions of the body of the cylinder.
     CRect bodyRect(pt.x, pt.y, pt.x + width, pt.y + height); 

     // Create a region based on the body dimenions
     CRgn rectRgn;
     rectRgn.CreateRectRgnIndirect(&bodyRect);

     // Create a bounding rectangle for the endcap
     CRect curveRect(pt.x, -dy, pt.x + width, dy);

     // Compute the total size of the "fill" rectangle, which is
     // gradient-filled.  If the eye is "above" the center, we
     // will see the top as an ellipse and the bottom as a curve
     // If the eye is "below" the center, we will see the top
     // as a curve and the bottom as an ellipse

     if(eyeAngle > 0)
        { /* has top */
         top.top = pt.y - dy;
         top.bottom = pt.y + dy;
         curveRect.top += pt.y + height;
         curveRect.bottom += pt.y + height;
        } /* has top */
     else
        { /* has bottom */
         bottom.top = pt.y + height - dy;
         bottom.bottom = pt.y + height + dy;
         curveRect.top += pt.y;
         curveRect.bottom += pt.y;
        } /* has bottom */

     // Create a clipping region for the curve.  This will be
     // used to "truncate" the rectangular area we fill to
     // just show the curved part
     CRgn curveRgn;

     // In GDI, the endpoint is up-to-but-not-including the
     // maximum right side.  The problem with this is that
     // the end cap will come out a bit short, so add 1 to
     // compensate for this feature
     if(::GetGraphicsMode(dc) != GM_ADVANCED)
        curveRect.right++; // allow for standard off-by-one issue

     // Now create a region that is an ellipse
     curveRgn.CreateEllipticRgnIndirect(&curveRect);
 
     // "OR" the region into the current body region to extend
     // it to include the elliptical area.  The parts of
     // the ellipse that overlap the rectangle don't matter;
     // the parts that extend beyond the body region will give
     // us the bottom curve or top curve that we need

     rectRgn.CombineRgn(&rectRgn, &curveRgn, RGN_OR);

     // Now apply any transformation to it that is necessary
     // Note that the clipping region is always done in device units
     // AND in the device coordinate space
     // This code does not handle scaling via SetViewportExt/SetWindowExt
     // but it would have to use LPtoDP on the elements of the
     // RGNDATA

     CRgn clipRgn;
     clipRgn.CreateRectRgn(0,0,0,0);
     clipRgn.CopyRgn(&rectRgn);

     // Obtain the current clipping region, and AND the
     // two regions together.  This way, the cylinder region
     // cannot exceed the clipping region already established
     // Note that if the clipping region were merely replaced,
     // we might overdraw something, such as a border
 
     //----------------
     //  left,pt.y
     //   |
     //   |      peak, pt.y
     //   |      |
     //   |      |      right, pt.y
     //   |      |      |
     //   v      v      v
     //     ___________ ______________
     //    /           \     dy
     //   +------+------+ ------------
     //   |\___________/| _____dy_____
     //   |      |      |
     //   |      |      |
     //   |      |      |
     //   |      *      |   * = (0,0)
     //   |      |      |
     //   | _____|_____ |_____________
     //   |/     |     \|     dy
     //   +------+------+ ------------
     //    \___________/______dy______
     //   ^      ^      ^
     //   |      |      |
     //   |      |      right, pt.y + height
     //   |      peak, pt.y + height
     //   left, pt.y + height


     TRIVERTEX tv[4] = {
        {   left,  pt.y - dy,
                                       MAKEWORD(0, GetRValue(darkColor)),
                                       MAKEWORD(0, GetGValue(darkColor)),
                                       MAKEWORD(0, GetBValue(darkColor)),
                                       MAKEWORD(0, 255)}, 
        {   peak,  pt.y + height + dy,
                                       MAKEWORD(0, GetRValue(lightColor)),
                                       MAKEWORD(0, GetGValue(lightColor)),
                                       MAKEWORD(0, GetBValue(lightColor)),
                                       MAKEWORD(0, 255)},
        {   peak,  pt.y - dy,
                                       MAKEWORD(0, GetRValue(lightColor)),
                                       MAKEWORD(0, GetGValue(lightColor)),
                                       MAKEWORD(0, GetBValue(lightColor)),
                                       MAKEWORD(0, 255)},
        {   right, pt.y + height + dy,
                                       MAKEWORD(0, GetRValue(darkColor)),
                                       MAKEWORD(0, GetGValue(darkColor)),
                                       MAKEWORD(0, GetBValue(darkColor)),
                                       MAKEWORD(0, 255)},
     };

     GRADIENT_RECT gr[2] = {
        {0, 1},
        {2, 3},
     };



     // Select the new clipping region and gradient-fill the
     // area.  The clipping region limits the gradient fill
     // so that the extended "spillover" of the left or right
     // will not actually draw
 
     { /* draw gradient */
      int save2 = dc.SaveDC();
 
      if(::GetGraphicsMode(dc) == GM_ADVANCED)
         { /* may need transform */
          XFORM Matrix;
          ::GetWorldTransform(dc, &Matrix);
          CPoint originalOffset( (int)Matrix.eDx, (int)Matrix.eDy);
          //  +-         -+
          //  |  M11  M12 |
          //  |  M21  M22 |
          //  |   dx   dy |
          //  +-         -+

          // To make this work under rotation we have to normalize it
          // to (0,0) so it rotates without displacements
          Matrix.eDx = 0.0f;
          Matrix.eDy = 0.0f;
          //  +-         -+
          //  |  M11  M12 |
          //  |  M21  M22 |
          //  |   0    0  |
          //  +-         -+ 

          // Now create a new region transformed by the matrix
          CRegionData data;
          data.Set(clipRgn);
          HRGN xformrgn = ::ExtCreateRegion(&Matrix, data.GetSize(), data.GetData());
          clipRgn.DeleteObject();
          clipRgn.Attach(xformrgn);

          CPoint offset = dc.GetViewportOrg();
          clipRgn.OffsetRgn(offset);
          clipRgn.OffsetRgn(originalOffset);
         } /* may need transform */
      else
         { /* non-transform */
          CPoint offset = dc.GetViewportOrg();
          clipRgn.CopyRgn(&rectRgn);
          clipRgn.OffsetRgn(offset);
         } /* non-transform */

      { /* clip to enclosing control */
       CRgn  existingClipRgn;
       existingClipRgn.CreateRectRgn(0,0,0,0);
       ::GetClipRgn((HDC)dc, (HRGN)existingClipRgn);

       clipRgn.CombineRgn(&clipRgn, &existingClipRgn, RGN_AND);
      } /* clip to enclosing control */

      dc.SelectClipRgn(&clipRgn);
 
      BOOL result = dc.GradientFill(tv, 4, gr, 2, GRADIENT_FILL_RECT_H);
      dc.RestoreDC(save2);
     } /* draw gradient */

     // Create a brush which is the "average" color for the end cap
     COLORREF endcap = RGB( (GetRValue(darkColor) + GetRValue(lightColor)) / 2,
                            (GetGValue(darkColor) + GetGValue(lightColor)) / 2,
                            (GetBValue(darkColor) + GetBValue(lightColor)) / 2);
     CBrush ecb(endcap);
     dc.SelectObject(&ecb);

     if(!top.IsRectEmpty())
        dc.Ellipse(&top);
     if(!bottom.IsRectEmpty())
        dc.Ellipse(&bottom);

     dc.RestoreDC(save);
    } // Cylinder::Draw
