// GradientButton.cpp : implementation file
//

#include "stdafx.h"
#include "CylinderApp.h"
#include "GradientButton.h"


// CGradientButton

IMPLEMENT_DYNAMIC(CGradientButton, CButton)

/****************************************************************************
*                      CGradientButton::CGradientButton
* Effect: 
*       Constructor
****************************************************************************/

CGradientButton::CGradientButton()
   {
    darkColor = RGB(0,0,0);
    lightColor = RGB(255, 255, 255);
   }

CGradientButton::~CGradientButton()
   {
   }


/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(CGradientButton, CButton)
END_MESSAGE_MAP()



// CGradientButton message handlers



/****************************************************************************
*                       CGradientButton::SetLightColor
* Inputs:
*       COLOREF c:
* Result: void
*       
* Effect: 
*       Sets the light color
****************************************************************************/

void CGradientButton::SetLightColor(COLORREF c)
    {
     lightColor = c;
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CGradientButton::SetLightColor

/****************************************************************************
*                        CGradientButton::SetDarkColor
* Inputs:
*       COLORREF c:
* Result: void
*       
* Effect: 
*       Sets the dark color
****************************************************************************/

void CGradientButton::SetDarkColor(COLORREF c)
    {
     darkColor = c;
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CGradientButton::SetDarkColor

/****************************************************************************
*                          CGradientButton::DrawItem
* Inputs:
*       LPDRAWITEMSTRUCT dis:
* Result: void
*       
* Effect: 
*       Draws the button
****************************************************************************/

void CGradientButton::DrawItem(LPDRAWITEMSTRUCT dis)
    {
     CRect r = dis->rcItem;
     CDC * dc = CDC::FromHandle(dis->hDC);
     CSize border(::GetSystemMetrics(SM_CXBORDER), ::GetSystemMetrics(SM_CYBORDER));
                                           // Dimensions of a border line
                                           // We use ::GetSystemMetrics so we work on
                                           // hi-res displays
     
     int save = dc->SaveDC();
     CRgn rgn;
     rgn.CreateRectRgnIndirect(&r);

     CPoint offset(0,0);
     if(!left)
        offset.x = -r.Width();

     // We deflate our useful area by the width of a
     // frame, which is twice a border. 
     //r.InflateRect(-2 * border.cx, -2 * border.cy);

     TRIVERTEX tv[2] = {
        { border.cx + offset.x, border.cy, 
                   MAKEWORD(0, GetRValue(darkColor)),
                   MAKEWORD(0, GetGValue(darkColor)),
                   MAKEWORD(0, GetBValue(darkColor)),
                   MAKEWORD(0, 255)}, 
        { border.cx + offset.x + r.Width()*2, border.cy + r.Height(),
                   MAKEWORD(0, GetRValue(lightColor)),
                   MAKEWORD(0, GetGValue(lightColor)),
                   MAKEWORD(0, GetBValue(lightColor)),
                   MAKEWORD(0, 255)}
     };

     GRADIENT_RECT gr[] = {0, 1};

     int save2 = dc->SaveDC();
//     dc->SelectClipRgn(&rgn);
//     dc->SetWindowOrg(offset);
     dc->GradientFill(tv, 2, gr, 1, GRADIENT_FILL_RECT_H);
     dc->RestoreDC(save2);
     

     if(dis->itemState & ODS_SELECTED)
        { /* down */
 //        dc->DrawEdge(&dis->rcItem, EDGE_SUNKEN, BF_RECT |  BF_SOFT);
        } /* down */
     else
        { /* up */
         dc->DrawEdge(&dis->rcItem, EDGE_RAISED, BF_RECT |  BF_SOFT);
        } /* up */

     dc->RestoreDC(save);
    }
