#include "stdafx.h"
#include "RegionData.h"


/****************************************************************************
*                            CRegionData::GetData
* Result: LPRGNDATA
*       The region data
****************************************************************************/

LPRGNDATA CRegionData::GetData()
    {
     return (LPRGNDATA)data.GetData();
    } // CRegionData::GetData

/****************************************************************************
*                            CRegionData::GetSize
* Result: DWORD
*       Length of the RGNDATA
****************************************************************************/

DWORD CRegionData::GetSize()
    {
     return (DWORD)data.GetSize();
    } // CRegionData::GetSize

/****************************************************************************
*                              CRegionData::Set
* Inputs:
*       CRgn & rgn: Region
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Captures the region data of the region
****************************************************************************/

BOOL CRegionData::Set(CRgn & rgn)
    {
     data.SetSize(0); // remove old data
     DWORD size = ::GetRegionData(rgn, 0, NULL);
     data.SetSize(size);
     size = ::GetRegionData(rgn, size, (LPRGNDATA)data.GetData());
     return size != 0;
    } // CRegionData::Set

/****************************************************************************
*                         CRegionData::GetHeaderSize
* Result: DWORD
*       Size of header
****************************************************************************/

DWORD CRegionData::GetHeaderSize()
    {
     if(data.GetSize() == 0)
        return 0;
     return ((RGNDATA*)data.GetData())->rdh.dwSize;
    } // CRegionData::GetHeaderSize

/****************************************************************************
*                            CRegionData::GetCount
* Result: DWORD
*       Number of rectangles in array
****************************************************************************/

DWORD CRegionData::GetCount()
    {
     if(data.GetSize() == 0)
        return 0;
     return ((RGNDATA*)data.GetData())->rdh.nCount;
    } // CRegionData::GetCount

/****************************************************************************
*                            CRegionData::GetBound
* Result: CRect *
*       Pointer to bound rectangle, NULL if not valid region
****************************************************************************/

CRect * CRegionData::GetBound()
    {
     if(data.GetSize() == 0)
        return NULL;
     return (CRect *)&(((RGNDATA*)data.GetData())->rdh.rcBound);
    } // CRegionData::GetBound

/****************************************************************************
*                       CRegionData::GetFirstRegionRect
* Result: POSITION
*       A position value; 0 (NULL) if not valid
* Notes:
*       The POSITION is offset by 1 so 0 will represent no data
****************************************************************************/

POSITION CRegionData::GetFirstRegionRect()
    {
     return (POSITION)1;
    } // CRegionData::GetFirstRegionRect

/****************************************************************************
*                       CRegionData::GetNextRegionRect
* Inputs:
*       POSITION & pos: Desired position
* Result: CRect *
*       A rectangle, or NULL if no more rectangles
****************************************************************************/

CRect * CRegionData::GetNextRegionRect(POSITION & pos)
    {
     if(data.GetSize() == 0)
        return NULL;
     UINT_PTR p = (UINT_PTR)pos;
     if(p == 0)
        return NULL;
     
     if(p - 1 >= ((RGNDATA*)data.GetData())->rdh.nCount)
        return NULL;
     pos = (POSITION)(p + 1);
     return (CRect*)&((RGNDATA*)data.GetData())->Buffer[p - 1];
    } // CRegionData::GetNextRegionRect

/****************************************************************************
*                          CRegionData::GetPosIndex
* Inputs:
*       POSITION p:
* Result: int
*       Integer value representing position
* Effect: 
*       
****************************************************************************/

int CRegionData::GetPosIndex(POSITION p)
    {
     return (int)(((INT_PTR)p) - 1);
    } // CRegionData::GetPosIndex
