/*****************************************************************************
*           Change Log
*  Date     | Change
*-----------+-----------------------------------------------------------------
* 30-Mar-08 | Created
* 30-Mar-08 | REQ #001: Courtest Hans-J. Ude, flicker removal (which I had been
*           | too lazy to implement...)
*****************************************************************************/
// Sample.cpp : implementation file
//

#include "stdafx.h"
#include "CylinderApp.h"
#include "Sample.h"


// CSample

IMPLEMENT_DYNAMIC(CSample, CStatic)

/****************************************************************************
*                              CSample::CSample
* Effect: 
*       Constructor
****************************************************************************/

CSample::CSample() 
   {
    UseMatrix = FALSE;
   }

CSample::~CSample()
   {
   }


/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(CSample, CStatic)
    ON_WM_PAINT()
    ON_WM_SIZE()
    ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/****************************************************************************
*                             CSample::Recompute
* Result: void
*       
* Effect: 
*       Recomputes absolute sizes
****************************************************************************/

void CSample::Recompute()
    {
     CRect r;
     GetClientRect(&r);

     int h = (int)((double)r.Height() * (double)height / 100.0);
     cyl.SetHeight(h);

     int w = (int)((double)r.Width() * (double)width / 100.0);
     cyl.SetWidth(w);
     
     cyl.SetLightingAngle(lightingAngle);
     cyl.SetEyeAngle(eyeAngle);
     Invalidate();
    } // CSample::Recompute

/****************************************************************************
*                             CSample::SetHeight
* Inputs:
*       int percent: Height in % of total size
* Result: void
*       
* Effect: 
*       Sets the height
****************************************************************************/

void CSample::SetHeight(int percent)
    {
     height = percent;
     Recompute();
    } // CSample::SetHeight

/****************************************************************************
*                              CSample::SetWidth
* Inputs:
*       int percent:
* Result: void
*       
* Effect: 
*       Sets the width
****************************************************************************/

void CSample::SetWidth(int percent)
    {
     width = percent;
     Recompute();
    } // CSample::SetWidth

/****************************************************************************
*                            CSample::SetEyeAngle
* Inputs:
*       int degrees:
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void CSample::SetEyeAngle(int degrees)
    {
     eyeAngle = degrees;
     Recompute();
    } // CSample::SetEyeAngle

/****************************************************************************
*                          CSample::SetLightingAngle
* Inputs:
*       int degrees:
* Result: 
*       
* Effect: 
*       
****************************************************************************/

void CSample::SetLightingAngle(int degrees)
    {
     lightingAngle = degrees;
     Recompute();
    } // CSample::SetLightingAngle

/****************************************************************************
*                              CSample::OnPaint
* Result: void
*       
* Effect: 
*       Draws the sample image
****************************************************************************/

void CSample::OnPaint()
    {
     CPaintDC ScreenDC(this); // device context for painting        // REQ #001
    
     CDC dc;                                                        // REQ #001
     CBitmap bm;                                                    // REQ #001

     CRect r;
     GetClientRect(&r);

     bm.CreateCompatibleBitmap(&ScreenDC, r.Width(), r.Height());   // REQ #001
     dc.CreateCompatibleDC(&ScreenDC);                              // REQ #001 

     int save = dc.SaveDC();
     dc.SelectObject(&bm);                                          // REQ #001

     dc.FillSolidRect(&r, ::GetSysColor(COLOR_3DFACE));

     CPoint pt;
     //*********************
     // |<--------r.width--------->|
     // |                          |
     // |       <---width--->      |
     // |      |##**--.--**##|     |
     // |      |##**--.--**##|     |
     // |      |##**--.--**##|     |
     //*********************
     // pt.x = r.Width()/2 - .5 * r.Width*widthpercent;
     // pt.y = r.Height()/2 - .5 * r.Height * heightpercent;

     pt.x = r.Width() / 2 - cyl.GetWidth() / 2;
     
     pt.y = r.Height() / 2 - cyl.GetHeight() / 2;
     
     CRgn rgn;
     CRgn client;
     
     if(UseMatrix)
        { /* use transform matrix */
         cyl.SetMatrix(&Matrix);
        } /* use transform matrix */
     else
        { /* no transformation */
         cyl.SetMatrix(NULL);
        } /* no transformation */

     client.DeleteObject();
     client.CreateRectRgnIndirect(&r);
     dc.SelectClipRgn(&client);

     cyl.SetPos(r.Width() / 2, r.Height() / 2);

     cyl.Draw(dc);

     ScreenDC.BitBlt(0, 0, r.Width(), r.Height(), &dc, 0, 0, SRCCOPY);// REQ #001
     
     dc.RestoreDC(save);
    }

/****************************************************************************
*                               CSample::OnSize
* Inputs:
*       UINT nType:
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Handles resize
****************************************************************************/

void CSample::OnSize(UINT nType, int cx, int cy)
    {
     CStatic::OnSize(nType, cx, cy);

     Recompute();
    }

/****************************************************************************
*                            CSample::OnEraseBkgnd
* Inputs:
*       CDC * PDC: DC to draw to
* Result: BOOL
*       
* Effect: 
*       Does nothing; avoids flash
****************************************************************************/

BOOL CSample::OnEraseBkgnd(CDC* pDC)
    {
     return TRUE;

     //return CStatic::OnEraseBkgnd(pDC);
    }

/****************************************************************************
*                           CSample::SetLightColor
* Inputs:
*       COLORREF c: New color to set
* Result: void
*       
* Effect: 
*       Sets the light color
****************************************************************************/

void CSample::SetLightColor(COLORREF c)
    {
     cyl.SetLightColor(c);
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CSample::SetLightColor

/****************************************************************************
*                           CSample::GetLightColor
* Result: COLORREF
*       Current light color
****************************************************************************/

COLORREF CSample::GetLightColor()
    {
     return cyl.GetLightColor();
    } // CSample::GetLightColor

/****************************************************************************
*                            CSample::SetDarkColor
* Inputs:
*       COLORREF c: new color to set
* Result: void
*       
* Effect: 
*       Sets the dark color
****************************************************************************/

void CSample::SetDarkColor(COLORREF c)
    {
     cyl.SetDarkColor(c);
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CSample::SetDarkColor

/****************************************************************************
*                            CSample::GetDarkColor
* Result: COLORREF
*       Dark color
****************************************************************************/

COLORREF CSample::GetDarkColor()
    {
     return cyl.GetDarkColor();
    } // CSample::GetDarkColor

/****************************************************************************
*                            CSample::EnableXFORM
* Inputs:
*       BOOL enable: desired mode
* Result: void
*       
* Effect: 
*       Enables/disables transform matrix
****************************************************************************/

void CSample::EnableXFORM(BOOL enable)
    {
     if(UseMatrix == enable)
        return;
     UseMatrix = enable;
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CSample::EnableXFORM

/****************************************************************************
*                             CSample::SetMatrix
* Inputs:
*       XFORM & M:
* Result: void
*       
* Effect: 
*       Updates the transform matrix
****************************************************************************/

void CSample::SetMatrix(XFORM & M)
    {
     Matrix = M;
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CSample::SetMatrix
